@@ -78,6 +78,7 @@ enum
   COL_CM,
   COL_PROTOCOL_NAME,
   COL_IS_GTALK,
+  COL_IS_FACEBOOK,
   COL_COUNT
 };
 
@@ -130,12 +131,16 @@ protocol_chooser_sort_func (GtkTreeModel *model,
        * non-gtalk */
       if (cmp == 0)
         {
-          gboolean is_gtalk;
+          gboolean is_gtalk, is_facebook;
           gtk_tree_model_get (model, iter_a,
             COL_IS_GTALK, &is_gtalk,
+            COL_IS_FACEBOOK, &is_facebook,
             -1);
 
-          cmp = is_gtalk ? 1 : -1;
+          if (is_gtalk || is_facebook)
+            cmp = 1;
+          else
+            cmp = -1;
         }
     }
 
@@ -167,6 +172,12 @@ protocol_choosers_add_cm (EmpathyProtocolChooser *chooser,
          */
         continue;
 
+      if (!tp_strdiff (cm->name, "haze") &&
+          !tp_strdiff (proto->name, "facebook"))
+        /* Facebook now supports XMPP so drop the purple facebook plugin; user
+         * should use Gabble */
+        continue;
+
       if (tp_strdiff (cm->name, "haze") && !tp_strdiff (saved_cm_name, "haze"))
         {
           GtkTreeIter titer;
@@ -220,6 +231,7 @@ protocol_choosers_add_cm (EmpathyProtocolChooser *chooser,
           COL_CM, cm,
           COL_PROTOCOL_NAME, proto->name,
           COL_IS_GTALK, FALSE,
+          COL_IS_FACEBOOK, FALSE,
           -1);
 
       if (!tp_strdiff (proto->name, "jabber") &&
@@ -233,6 +245,18 @@ protocol_choosers_add_cm (EmpathyProtocolChooser *chooser,
              COL_CM, cm,
              COL_PROTOCOL_NAME, proto->name,
              COL_IS_GTALK, TRUE,
+             COL_IS_FACEBOOK, FALSE,
+             -1);
+
+          display_name = empathy_protocol_name_to_display_name ("facebook");
+          gtk_list_store_insert_with_values (priv->store,
+             NULL, 0,
+             COL_ICON, "im-facebook",
+             COL_LABEL, display_name,
+             COL_CM, cm,
+             COL_PROTOCOL_NAME, proto->name,
+             COL_IS_GTALK, FALSE,
+             COL_IS_FACEBOOK, TRUE,
              -1);
         }
 
@@ -283,7 +307,8 @@ protocol_chooser_constructed (GObject *object)
           G_TYPE_STRING,    /* Label     */
           G_TYPE_OBJECT,    /* CM */
           G_TYPE_STRING,    /* protocol name  */
-          G_TYPE_BOOLEAN);  /* is gtalk  */
+          G_TYPE_BOOLEAN,   /* is gtalk  */
+          G_TYPE_BOOLEAN);  /* is facebook  */
 
   /* Set the protocol sort function */
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (priv->store),
@@ -396,12 +421,13 @@ protocol_chooser_filter_visible_func (GtkTreeModel *model,
   TpConnectionManager *cm = NULL;
   gchar *protocol_name = NULL;
   gboolean visible = FALSE;
-  gboolean is_gtalk;
+  gboolean is_gtalk, is_facebook;
 
   gtk_tree_model_get (model, iter,
       COL_CM, &cm,
       COL_PROTOCOL_NAME, &protocol_name,
       COL_IS_GTALK, &is_gtalk,
+      COL_IS_FACEBOOK, &is_facebook,
       -1);
 
   if (cm != NULL && protocol_name != NULL)
@@ -413,7 +439,7 @@ protocol_chooser_filter_visible_func (GtkTreeModel *model,
 
       if (protocol != NULL)
         {
-          visible = priv->filter_func (cm, protocol, is_gtalk,
+          visible = priv->filter_func (cm, protocol, is_gtalk, is_facebook,
               priv->filter_user_data);
         }
     }
@@ -439,7 +465,8 @@ TpConnectionManager *
 empathy_protocol_chooser_dup_selected (
     EmpathyProtocolChooser *protocol_chooser,
     TpConnectionManagerProtocol **protocol,
-    gboolean *is_gtalk)
+    gboolean *is_gtalk,
+    gboolean *is_facebook)
 {
   GtkTreeIter iter;
   TpConnectionManager *cm = NULL;
@@ -478,6 +505,13 @@ empathy_protocol_chooser_dup_selected (
               COL_IS_GTALK, is_gtalk,
               -1);
         }
+
+      if (is_facebook != NULL)
+        {
+          gtk_tree_model_get (GTK_TREE_MODEL (cur_model), &iter,
+              COL_IS_FACEBOOK, is_facebook,
+              -1);
+        }
     }
 
   return cm;
